/**\
       REXX background BBS maintainance proccessor.

       Sleeps until a flag is found in the session message queue and then
       carries out any particular task in the background. Once the
       operation is complete, sleeps again until the next flag arrives.

       Recommend calling it using  'Priority 2 1' so that it doesn't compete
       with front end BBS activities

       Written by Brenda Moon & M.N.Stephenson.
\**/

parse source . . Spec

/*      Set this variable to <>0 to run this program in test mode */
TestModeFlag  = 0

if rxfuncquery(sysloadfuncs) \= 0 then
  do
    call rxfuncadd 'sysloadfuncs','rexxutil','sysloadfuncs'
    call sysloadfuncs
  end

/*      Constants,      */
False    = 0
True     = 1
Exist           = 1
NotExist        = 0
StartTime      = time()
StartDate      = date()
Message         = '(none)'
Tab             = d2c(9)
Esc = d2c(27)||'['

ProgramName    = Esc||'1;33m'||filespec('NAME', Spec)||Esc||'0m'

CurrentDir     = directory()

Version         = 'Version 1.07'
DateModified    = '(10/6/1994)'

SpaceReq = 3000000    /* 3 Mb */
SpaceLow = False

/*      Perform test for error log file path test,       */
LogDir         = 'D:\BINKLEY\LOGS'

if directory(LogDir) <> LogDir then do
   say directory(LogDir)||Tab||LogDir
   say
   say Tab||' *******CANNOT CREATE LOG FILE*******'
   say Tab||' *******ABORTING '||ProgramName||'*******'
   exit(1)
end

/**\
 PATHS,
      NOTE TO USER: The function CheckPath('[Drive]:\[Path]') tests
                    for the existence of '[Drive]:\[Path]' and sets
                    the appropriate variable to it ONLY IF IT DOES EXIST.
                    Therefore, YOU must edit the '[Drive]:\[Path]' arguments
                    of each call to CheckPath to reflect YOUR hard disk.

               --MAKE A COPY OF THIS FILE BEFORE YOU EDIT IT--
\**/
BinkleyDir     = CheckPath('D:\BINKLEY')
NetfileDir     = CheckPath('C:\NETFILE')
SquishDir      = CheckPath('D:\SQUISH')
MaxDir         = CheckPath('D:\MAX')
FileListDir    = CheckPath('C:\FILE\UTIL')
NodelistDir    = CheckPath('D:\BINKLEY\NODELIST')

TossFile       = MaxDir||'\ECHOTOSS.LOG'     /* Can't check if this exists */
                                             /* as it MAY NOT at startup   */

/**\
FLAGS,
       NOTE TO USER: This REXX program is designed to be run continuously
                     as a background process. Other forground processes
                     can send messages to this program in order for it to
                     perform some 'house-keeping' task(s). This is achieved
                     by using the OS/2 message queue set up by the function
                     CheckQueue('[queue name]') Any number of messages can
                     be sent to this program, the processing of which RELIES
                     upon,
                       a) including some code to perform the desired task, and
                       b) registering the message within this program.

                     The stem variable 'Flag.[x]' - see below contains any
                     message used by this program. The first element in this
                     stem variable, 'Flag.0' holds the number of registered
                     messages. All messages are converted to upper case.

               --MAKE A COPY OF THIS FILE BEFORE YOU EDIT IT--
\**/
FlagQueue      = CheckQueue('BBS_CMD')

Flag.0          = 7
Flag.1          = 'QUIT'                /* Terminate this program       */
Flag.2          = 'SQUISH'              /* Run SQUISH to process mail   */
Flag.3          = 'FB'                  /* Run FBP - File Base Update   */
Flag.4          = 'LCALL'               /* Run LCALLP - Todays callers update */
Flag.5          = 'FILES'               /* Produce new version of file lists */
Flag.6          = 'LOGPACK'             /* Truncate log files */
Flag.7          = 'SQPACK'              /* Run SQPACK to compress message areas */

do c = 1 to Flag.0
   Calls.c      = 0
   LastRun.c    = ''
end

NumVoidFlags  = 0
VoidFlags.0    = ''

do forever

   if queued() = 0 then do
      call SysCls
      call SysCurState 'OFF'

      call DisplayStatus

      do while queued() = 0
         call SysSleep 1

         if chars('STDIN') > 0 then do
            chr = SysGetKey('NOECHO')
            if chr = d2c(27) then call EndProgram
         end  /* Do */

      end  /* Do while */

      call SysCls

   end  /* Do */


   /* REMARK OUT THIS LINE TO DISABLE SESSION QUEUE RATIONALISATION */
   call RationaliseQueue

   pull Message

   select

      when Message = Flag.1 then do          /*      QUIT            */
         Calls.1     = Calls.1 + 1
         LastRun.1   = date()||'  '||time()
         call EndProgram
      end  /* Do */

      when Message = Flag.2 then do          /*      SQUISH          */
         Calls.2     = Calls.2 + 1
         LastRun.2   = date()||'  '||time()  
         if TestModeFlag = 1 then
            call TestRoutine Message
         else
            if SpaceReq > DiskSpace(MaxDir) then  /* only process mail if  */
                SpaceLow = True                   /* there is enough space */
            else do
                SpaceLow = False
                call ProcessMail
            end
      end  /* Do */

      when Message = Flag.3 then do          /*      FB              */
         Calls.3     = Calls.3 + 1
         LastRun.3   = date()||'  '||time()
         if TestModeFlag = 1 then
            call TestRoutine Message
         else
            call UpdateFileBase
      end  /* Do */

      when Message = Flag.4 then do          /*      LCALL           */
         Calls.4     = Calls.4 + 1
         LastRun.4   = date()||'  '||time()
         if TestModeFlag = 1 then
            call TestRoutine Message
         else
            call UpdateTodaysCallers
      end  /* Do */

      when Message = Flag.5 then do          /*      FILES           */
         Calls.5     = Calls.5 + 1
         LastRun.5   = date()||'  '||time()
         if TestModeFlag = 1 then
            call TestRoutine Message
         else
            call UpdateFileList
      end  /* Do */

      when Message = Flag.6 then do          /*      LOGPACK           */
         Calls.6     = Calls.6 + 1
         LastRun.6   = date()||'  '||time()
         if TestModeFlag = 1 then
            call TestRoutine Message
         else
            call TruncateLogFiles
      end  /* Do */

      when Message = Flag.7 then do          /*      SQPACK           */
         Calls.7     = Calls.7 + 1
         LastRun.7   = date()||'  '||time()
         if TestModeFlag = 1 then
            call TestRoutine Message
         else
            if DiskSpace(MaxDir) > 2000000 then /* only pack if have 2Mb */
                 call MessagePack
      end  /* Do */

      otherwise
      /* Destroy unrecognised messages from the message queue   */

      occ       = NotExist

      do c = 0 to NumVoidFlags
         if VoidFlags.c = Message then occ = Exist
      end /* do */

      if occ = NotExist then do
         NumVoidFlags = NumVoidFlags + 1
         VoidFlags.NumVoidFlags   = Message
      end

   end  /* select */

end /* Do Forever */

exit 1

/**\
       Sub-routine: ErrorProc
       This routine writes the passed argument to [name of program].LOG
\**/
ErrorProc: Procedure expose Spec LogDir Tab ProgramName CurrentDir
parse arg ErrorMsg

pName   = filespec('NAME', Spec)

tName   = left(pName, pos('.', pName) - 1)
ErrorLogFile = LogDir||'\'||tName||'.LOG'

rc      = beep(880, 1000)

say; say ' *******'||errormsg||'*******'
say; say ' LOGGING ERROR TO '||ErrorLogFile

call lineout ErrorLogFile, ErrorMsg||Tab||date()||Tab||time()
call lineout ErrorLogFile

say ' Terminating '||ProgramName

rc      = directory(CurrentDir)
exit(1)

/**\
       Sub-routine CheckPath
       This routine accepts a path argument and tests
       for its existance.
\**/
CheckPath: procedure
parse arg PathName

if directory(PathName) = '' then
   call ErrorProc 'Path error: '||pathname||' does not exist!'

return PathName
exit 1

/**\
       Sub-routine CheckQueue
       This routine accepts the name of the message queue used to
       pass flags to this program. The queue is created if needed and
       set as the current session message queue.
\**/
CheckQueue: procedure
parse arg QueueName

QueueTest      = RxQueue('CREATE', QueueName)

if QueueTest <> QueueName then
   rc   = RxQueue('DELETE', QueueTest)

OldQueue       = RxQueue('SET', QueueName)

return QueueName
exit 1

/**\
     DiskSpace:

\**/
DiskSpace: procedure

arg Path

parse value SysDriveInfo(filespec('DRIVE', Path)) with . Space . .

return Space

exit 1
/**\
       Sub-routine: DisplayStatus
       Displays status information
\**/
DisplayStatus: 

call DisplayTitle
say
call charout ,' 'Esc||'1;37;46m Activated: '||Esc||'0m'

call charout ,Tab||StartDate||'  '||StartTime

if TestModeFlag <> 0 then
   call charout ,Tab||Esc||'1;33mRUNNING IN TEST MODE'||Esc||'0m'
else
   call charout ,Tab||'Running in normal mode.'

say
say
say
if SpaceLow then
  say ' 'Esc||'1;37;46m Registered messages: '||Esc||'0m         '||Esc||'1;31m Warning Space Low!'||Esc||'0m'
else
  say ' 'Esc||'1;37;46m Registered messages: '||Esc||'0m'

say
do c = 1 to flag.0
   call charout ,Esc||'1;36m'||Tab||Calls.c||Esc||'0m '
   say 'calls for'||Tab||Flag.c||Tab||LastRun.c
end
say
say
say ' Press'||Esc||'1;31m ESC'||Esc'0m to terminate.'
say

return
exit 1

/**\
       Sub-routine: DisplayTitle
       Displays program title line
\**/
DisplayTitle:
say
VertLine = Esc||'0;30;46m'||Esc||'1;37;46m'
call charout , ' '||Esc||'1;46m '||ProgramName||Esc||'1;46m '||VertLine
parse value SysCurPos() with . cols
call charout , center(Version||'  '||DateModified, 79 - 23 - cols)
call charout , VertLine||' Brenda Moon & M.N.S. '||Esc||'0m'
say

return
exit 1

/**\
       RationaliseQueue
       Session queue rationalisation. Removes multiple flags from the
       queue, while preserving the order of the flags.
\**/
RationaliseQueue: procedure

Rationalised = 0

do while queued() > 0

   pull Message

   if Message = Flag.1 then do
      Calls.1 = Calls.1 + 1
      say ' called from within RationaliseQueue'
      call EndProgram
   end  /* Do */

   occ = NotExist

   do r = 0 to Rationalised
      if Rz.r = Message then occ = Exist
   end /* do */

   if occ = NotExist then do
      Rationalised = Rationalised + 1
      Rz.Rationalised = Message
   end

end  /* Do */

do r = 1 to Rationalised
   push Rz.r
end /* do */

return
/**\
       Sub-routine: TestRoutine
       This routine is executed when the variable TestModeFlag = 1
\**/
TestRoutine: Procedure expose ProgramName Tab
parse arg TestMessage

call SysCls
rc = beep(880,50)

say
say ' '||ProgramName||' running in test mode.'
say Tab||'Queue message:'||Tab||TestMessage
say

call SysSleep 2
rc = beep(1660,50)

return
exit

/**\
       Sub-routine:    EndProgram
       task:           Orderly termination of this CMD file
\**/
EndProgram:

if NumVoidFlags > 0 then
   do
      call SysCls
      say
      call DisplayTitle
      say
      say ' Unregistered message(s) posted in '||ProgramName||' session queue,'
      say
      do c = 1 to NumVoidFlags
         rc = beep(1660,50)
         say Tab||VoidFlags.c
      end
      say
   end
   else do
      call SysCls
      call DisplayStatus
   end  /* Do */

rc = beep(1660, 50)
call charout ,' '||ProgramName||Esc||'1;31m terminated.'||Esc||'0m'
say

rc = directory(CurrentDir)

exit

/**\
       Sub-routine: ProcessMail
       This code contains the specific actions to be performed
       when the flag.1 is detected,
\**/
ProcessMail:
Procedure expose SquishDir TossFile MaxDir Exist NotExist BinkleyDir NetfileDir NodelistDir LogDir

call SysCls

rc  = Directory(SquishDir)

/*      squish instructions...        */
'SQ386P -c'||SquishDir||'\squish.cfg -f'||TossFile||' in out squash'
if rc = 3 | rc = 4 then do /* errorlevel 3 or 4 means netmail */


/* if netmail area is *.msg should run SCANBLD here */

   NetMail = Exist

end /* do */
else Netmail = NotExist

'SQLINKP -c'||SquishDir||'\squish.cfg -f'||TossFile||' -k'
if stream(TossFile, 'command', 'query exist') \= '' then
        do
                'SQ386P -f'||TossFile||' link'
                if  sysfiledelete(TossFile) \= 0 then
                do
                        beep(880, 500)
                        say ' error: '||TossFile||' does not exist!'
                end
        end

rc = directory(BinkleyDir)

/*      check for the existance of *.tic files       */
rc      = sysfiletree(NetfileDir||'\*.tic' , files, 'f')

if rc <> 0 then call ErrorProc 'Error calling SysFileTree rc= '||rc

if files.0 > 0 then
do
        TicFiles = Exist
        say; say ' *.tic files have been found...'
        'TOUCH '||NetfileDir||'\*'
end
else TicFiles = NotExist

/*      tick file and yaticp addressed net mail,        */
if (NetMail = exist) | (TicFiles = exist) then do

'NETMGRP'       /* use net mail manager to move yatman messages  */
                /* to netyatic                                   */
'YATICP'

end /* do */

/*      check for yaticp made message packets,  */
rc  = sysfiletree(NetfileDir||'\*.pkt' , Files, 'f')
if rc <> 0 then call ErrorProc 'Error calling SysFileTree rc= '||rc
if Files.0 > 0 then
do
        say; say ' Squishing yatic message packets...'
        'SQUISHP -c'||SquishDir||'\squish.cfg -f'||TossFile||' in out squash'
end

if ticfiles = exist then call UpdateFileBase

Call UpdateNodelist   /* only does work if a nodediff exists to process */

return

exit 1

/**\
       Sub-routine:    UpdateFileBase
       task:           Run FBP to update the Maximus files Database
\**/
UpdateFileBase: procedure expose MaxDir

rc      = directory(MaxDir)
say; say ' Executing the file base processor.'
'FBP'

return

/**\
       Sub-routine:    UpdateNodelist
       task:           Run Fastlst to update Nodelist
\**/
UpdateNodelist: procedure expose NodelistDir NetfileDir LogDir

rc = sysfiletree(NetfileDir||'\nodediff.*','file','FO')
if rc <> 0 then call ErrorProc 'Error calling SysFileTree rc= '||rc

if file.0 > 0 then do /* there are some Nodediff files to process */

   rc= Directory(NodelistDir)

   'del '||NodelistDir||'\nodediff.*'
   'copy '||NetfileDir||'\nodediff.* '||NodelistDir

   do i=1 to file.0
      FileName = filespec("NAME", file.i)
      if translate(substr(FileName, pos('.', FileName) + 1,1)) = 'L' then do
         'lh x '||FileName
      end /* if */
   end /* do */

   'fastlstp -l'||LogDir||'\fastlst.log'

   'del '||NetfileDir||'\nodediff.*'

   /* create nodelist.lzh */
   'del nodelist.lzh'
   'lh a nodelist nodelist.1* nodelist.2* nodelist.3* nodelist.0*'
end /* if */

return

exit 1


/**\
       Sub-routine:    UpdateTodaysCallers
       task:           Run LCALLP to update the Todays Callers list
\**/
UpdateTodaysCallers: procedure expose MaxDir

rc      = directory(MaxDir)
say; say ' Executing LCALLP.'
'lcallp user.bbs '||MaxDir||'\misc\lastcall.bbs "Bunyip''s Cave BBS "'

return

exit 1

/**\
       Sub-routine:    UpdateFileList
       task:           Run FM (fileminder) to update download counters 
                       and duplicate files lists and then
                       run ALGEN to generate new Allfiles & NewFiles lists
\**/
UpdateFileList: procedure expose MaxDir FileListDir LogDir BinkleyDir

/* run WIMM to keep my mail for me in a seperate mail area */
'WIMMP -c'||BinkleyDir||'\wimm.cfg'

'C:\filemind\fm MAXFILES'
'C:\filemind\fm SCAN'
'C:\filemind\fm COUNTS SET'
'C:\filemind\fm LIST -d -rd8>C:\file\dupes\dupes8.txt'
'C:\filemind\fm LIST -d -rd7>C:\file\dupes\dupes7.txt'
rc = directory(MaxDir)

/* configure ALGEN to create "temporary" files and then copy them over "real" ones. */
'ALGEN'
rc = directory(FileListDir)

'LH a bunyip.lzh bunyip.lst'
'LH a newfiles.lzh newfiles.txt'


return

exit 1

/**\
       Sub-routine:    TruncateLogFiles
       task:           Run MAXLOG to truncate various log files
\**/
TruncateLogFiles: procedure expose MaxDir

rc = directory(MaxDir)

'MAXLOG 7 '||LogDir||'\yatic.log'
'MAXLOG 7 '||LogDir||'\bunyip0.log'
'MAXLOG 7 '||LogDir||'\squish.log'
'MAXLOG 7 '||LogDir||'\fm.log'
'MAXLOG 7 '||LogDir||'\wimm.log'
return

exit 1

/**\
       Sub-routine:    MessagePack
       task:           Run SQPACK to compress the Maximus message areas
\**/
MessagePack: procedure expose SquishDir MaxDir

rc      = directory(SquishDir)
say; say ' Executing SqPack.'
'SQPACKP '||MaxDir||'\area.dat'

return

