/**\
       IBM OS/2 2.x REXX file for BBS automation.

       Written by Brenda Moon and M.N.Stephenson

       Used to run Binkley for OS/2 as a front end to Maximus for OS/2.
       Includes handling of Binkley events, Mail and BBS callers.

\**/
parse source . . Spec

/*      Set this variable to <>0 to run this program in test mode */
TestModeFlag  = 0

if rxfuncquery(sysloadfuncs) \= 0 then
  do
    call rxfuncadd 'sysloadfuncs','rexxutil','sysloadfuncs'
    call sysloadfuncs
  end

/*      Constants,      */
False    = 0
True     = 1
Exist           = 1
NotExist        = 0
StartTime      = time()
StartDate      = date()
Message         = '(none)'
Tab             = d2c(9)
Esc = d2c(27)||'['

ProgramName    = Esc||'1;33m'||filespec('NAME', Spec)||Esc||'0m'

CurrentDir     = directory()

Version         = 'Version 1.04'
DateModified    = '(23/5/94)'

call SysCls
call SysCurState 'OFF'
call DisplayTitle

say
if TestModeFlag <> 0 then
   call charout ,Tab||d2c(27)||'[1;33mRUNNING IN TEST MODE'||d2c(27)||'[0m'
else
   call charout ,Tab||'Running in normal mode.'
say

/*      Perform test for error log file path test,       */
LogDir         = 'D:\BINKLEY\LOGS'

if directory(LogDir) <> LogDir then do
   say directory(LogDir)||Tab||LogDir
   say
   say Tab||' *******CANNOT CREATE LOG FILE*******'
   say Tab||' *******ABORTING '||ProgramName||'*******'
   exit(1)
end

/**\
 PATHS,
      NOTE TO USER: The function CheckPath('[Drive]:\[Path]') tests
                    for the existence of '[Drive]:\[Path]' and sets
                    the appropriate variable to it ONLY IF IT DOES EXIST.
                    Therefore, YOU must edit the '[Drive]:\[Path]' arguments
                    of each call to CheckPath to reflect YOUR hard disk.

               --MAKE A COPY OF THIS FILE BEFORE YOU EDIT IT--
**/
BinkleyDir      = CheckPath('D:\BINKLEY')
SquishDir       = CheckPath('D:\SQUISH')
MaxDir         = CheckPath('D:\MAX')

FlagQueue = CheckQueue('BBS_CMD')

do forever

   rc = directory(BinkleyDir)

   'BTP Share'

   if TestModeFlag = 0 then do
      select
         when rc = 1 then do
             call ExitProgram
             exit 0
         end

         when rc = 30 then call SendQMsg 'SQUISH'

         when rc = 17 then call DoSquish 'Poll_Hold'

         when rc = 16 then call DoSquish 'Poll_Paul'

         when rc = 11 then call DoSquish 'Poll_Andy'

         when rc = 14 then call Maintenance

      otherwise
         nop
      end  /* select */

   end  /* Do IF */
   else
      call TestRoutine rc

end /* do forever */

exit 1

/**\
       Sub-routine: DisplayTitle
       Displays program title line
\**/
DisplayTitle:
say
VertLine = Esc||'0;30;46m'||Esc||'1;37;46m'
call charout , ' '||Esc||'1;46m '||ProgramName||Esc||'1;46m '||VertLine
parse value SysCurPos() with . cols
call charout , center(Version||'  '||DateModified, 79 - 23 - cols)
call charout , VertLine||' Brenda Moon & M.N.S. '||Esc||'0m'
say

return
exit 1

/**\
       SendQMsg
       Pushes a message into the queue
\**/
SendQMsg: procedure
parse arg Message

call RxQueue 'SET', 'BBS_CMD'
queue Message

call SysSleep 1

return

exit 1

/**\
       Sub-routine CheckPath
       This routine accepts a path argument and tests
       for its existance.
\**/
CheckPath: procedure
parse arg PathName

if directory(PathName) = '' then
   call ErrorProc 'Path error: '||pathname||' does not exist!'

return PathName
exit 1

/**\
       Sub-routine CheckQueue
       This routine accepts the name of the message queue used to
       pass flags to this program. The queue is created if needed and
       set as the current session message queue.
\**/
CheckQueue: procedure
parse arg QueueName

QueueTest      = RxQueue('CREATE', QueueName)

if QueueTest <> QueueName then
   rc   = RxQueue('DELETE', QueueTest)

OldQueue       = RxQueue('SET', QueueName)

return QueueName
exit 1

/**\
       Sub-routine: ErrorProc
       This routine writes the passed argument to [name of program].LOG
\**/
ErrorProc: Procedure expose Spec LogDir Tab ProgramName CurrentDir
parse arg ErrorMsg

pName   = filespec('NAME', Spec)

tName   = left(pName, pos('.', pName) - 1)
ErrorLogFile = LogDir||'\'||tName||'.LOG'

rc      = beep(880, 1000)

say; say ' *******'||errormsg||'*******'
say; say ' LOGGING ERROR TO '||ErrorLogFile

call lineout ErrorLogFile, ErrorMsg||Tab||date()||Tab||time()
call lineout ErrorLogFile

say ' Terminating '||ProgramName

rc      = directory(CurrentDir)
exit(1)

/**\
     ExitProgram
     Performs all cleanup and displays session record.
\**/
ExitProgram:
Procedure expose CurrentDir ProgramName Version DateModified StartDate StartTime Esc Tab

rc = directory(CurrentDir)

beep(1660, 70)

call SysCls
call SysCurState 'OFF'
call DisplayTitle
say
say Tab||'Activated:'||Tab||StartDate||d2c(9)||StartTime
say Tab||'Terminated:'||Tab||date()||Tab||time()
say

say ' '||ProgramName||Esc||'1;31m terminated.'||Esc||'0m'

return

exit 1


/*\*
       TestRoutine
       This routine is executed when the variable TestModeFlag = 1
\**/
TestRoutine: Procedure expose ProgramName Tab
parse arg Message

call SysCls
rc = beep(880,50)

say
say ' '||ProgramName||' Test mode.'
say Tab||'ErrorLevel from Binkley:'||Tab||Message

call SysSleep 2
rc = beep(1660,50)

return

exit 1

/**\
        Sub-routine:   Maintenance
        Task: Perform daily Maintenance tasks
\**/
Maintenance: 
call SendQMsg 'FILES'
call SendQMsg 'LOGPACK'
call SendQMsg 'SQPACK'
return

exit 1


/**\
       DoSquish
       Use SQUISH to create a poll...
       I do this in foreground so that when Binkley restarts it
       immediately sees the poll.
\**/
DoSquish: procedure expose SquishDir
parse arg Message

rc = directory(SquishDir)
'squishp squash -c'||SquishDir||'\squish.cfg -s'||Message

return
exit 1

