/**\
  BBSLINE.Cmd

  Call Maximus in WFC mode for line given as command line arguement

  Usage:
       BBSLINE PortNo

   where 'PortNo' is the comm port number to use ( 1 to 4 )

  Notes:  1.  I use the same Task number as port number
          2.  Sets comm port to 19200

\**/
parse arg PortNo
parse source . . Spec

if rxfuncquery(sysloadfuncs) \= 0 then
  do
    call rxfuncadd 'sysloadfuncs','rexxutil','sysloadfuncs'
    call sysloadfuncs
  end

/*      Constants,      */
False    = 0
True     = 1
Exist           = 1
NotExist        = 0
StartTime      = time()
StartDate      = date()
Message         = '(none)'
Tab             = d2c(9)
Esc = d2c(27)||'['

ProgramName    = Esc||'1;33m'||filespec('NAME', Spec)||Esc||'0m'
Version         = 'Version 0.92'
DateModified    = '(22/11/93)'

CurrentDir     = directory()


if PortNo < 1 | PortNo > 4 then signal ParmError

/*      Perform test for error log file path test,       */
LogDir         = 'D:\BINKLEY\LOGS'

if directory(LogDir) <> LogDir then do
   say directory(LogDir)||Tab||LogDir
   say
   say Tab||' *******CANNOT CREATE LOG FILE*******'
   say Tab||' *******ABORTING '||ProgramName||'*******'
   exit(1)
end


call syscls
call DisplayTitle

/**\
 PATHS,
      NOTE TO USER: The function CheckPath('[Drive]:\[Path]') tests
                    for the existence of '[Drive]:\[Path]' and sets
                    the appropriate variable to it ONLY IF IT DOES EXIST.
                    Therefore, YOU must edit the '[Drive]:\[Path]' arguments
                    of each call to CheckPath to reflect YOUR hard disk.

               --MAKE A COPY OF THIS FILE BEFORE YOU EDIT IT--
**/
MaxDir          = CheckPath('D:\MAX')

FlagQueue = CheckQueue('BBS_CMD')

LogFile = LogDir||'\Bunyip'||PortNo||'.log'

message = '* '||left(date(),Length(date())-5) time()||' BBSLINE '
message = message||'WFC mode, n=0'||PortNo||' p='||PortNo
call lineout LogFile, Message
call lineout LogFile

rc = Directory(MaxDir)

MaxRet = 0
do while MaxRet <> 1

   MaxParams = ' -w -p'||PortNo||' -b19200 -s19200 -n0'||PortNo||' -l'||LogFile
   MaxParams = MaxParams||' -z\pipe\bbs'||PortNo

   'maxp.exe '||MaxParams
   MaxRet = rc
   if MaxRet = 12 | MaxRet = 11 then call SendQMsg 'SQUISH'

   call SendQMsg 'LCALL'
end

call ExitProgram
exit 0

/**\
       Sub-routine: DisplayTitle
       Displays program title line
\**/
DisplayTitle:
say
VertLine = Esc||'0;30;46m'||Esc||'1;37;46m'
call charout , ' '||Esc||'1;46m '||ProgramName||Esc||'1;46m '||VertLine
parse value SysCurPos() with . cols
call charout , center(Version||'  '||DateModified, 79 - 23 - cols)
call charout , VertLine||' Brenda Moon & M.N.S. '||Esc||'0m'
say

return
exit 1

/**\
       Sub-routine CheckQueue
       This routine accepts the name of the message queue used to
       pass flags to this program. The queue is created if needed and
       set as the current session message queue.
\**/
CheckQueue: procedure
parse arg QueueName

QueueTest      = RxQueue('CREATE', QueueName)

if QueueTest <> QueueName then
   rc   = RxQueue('DELETE', QueueTest)

OldQueue       = RxQueue('SET', QueueName)

return QueueName
exit 1

/**\
       SendQMsg
       Pushes a message into the queue
\**/
SendQMsg: procedure
parse arg Message

call RxQueue 'SET', 'BBS_CMD'
queue Message

call SysSleep 1

return

exit 1

/**\
       Sub-routine CheckPath
       This routine accepts a path argument and tests
       for its existance.
\**/
CheckPath: procedure
parse arg PathName

if directory(PathName) = '' then
   call ErrorProc 'Path error: '||pathname||' does not exist!'

return PathName
exit 1

/**\
       Sub-routine: ErrorProc
       This routine writes the passed argument to [name of program].LOG
\**/
ErrorProc: Procedure expose Spec LogDir Tab ProgramName CurrentDir
parse arg ErrorMsg

pName   = filespec('NAME', Spec)

tName   = left(pName, pos('.', pName) - 1)
ErrorLogFile = LogDir||'\'||tName||'.LOG'

rc      = beep(880, 1000)

say; say ' *******'||errormsg||'*******'
say; say ' LOGGING ERROR TO '||ErrorLogFile

call lineout ErrorLogFile, ErrorMsg||Tab||date()||Tab||time()
call lineout ErrorLogFile

say ' Terminating '||ProgramName

rc      = directory(CurrentDir)
exit(1)

/**\
     ExitProgram
     Performs all cleanup and displays session record.
\**/
ExitProgram:
Procedure expose CurrentDir ProgramName Version DateModified StartDate StartTime Esc Tab

rc = directory(CurrentDir)

beep(1660, 70)

call SysCls
call SysCurState 'OFF'
call DisplayTitle
say
say Tab||'Activated:'||Tab||StartDate||d2c(9)||StartTime
say Tab||'Terminated:'||Tab||date()||Tab||time()
say

say ' '||ProgramName||Esc||'1;31m terminated.'||Esc||'0m'

return

exit 1


/**\
 ParmError   Called when there is a command line parameter error

\**/
ParmError:
say ''
say 'Command line parameter error'
say ''
say 'Usage:'
say '     BBSLINE PortNo'
say ''
say '     PortNo can be 1, 2, 3 or 4'
exit 1

