/**\
       IBM OS/2 2.x REXX file for BBS automation.

       Called by Binkley "BBS Spawn" to run Maximus BBS

       Written by Brenda Moon and M.N.Stephenson

       Usage:    SpawnBBS Speed Baud Port TimeLeft ExtInfo

       Where    Speed    = DTE speed  (Modem to Computer)
                Baud     = DCE speed  (Modem to Modem)
                Port     = File Handle for Comm port (NOT PortNo)
                TimeLeft = Time until next Forced Binkley event
                ExtInfo  = Extended connection information (eg /ARQ)
\**/
parse source . . Spec
parse arg Speed Baud Port TimeLeft ExtInfo

if rxfuncquery(sysloadfuncs) \= 0 then
  do
    call rxfuncadd 'sysloadfuncs','rexxutil','sysloadfuncs'
    call sysloadfuncs
  end

/*      Constants,      */
False    = 0
True     = 1
Exist           = 1
NotExist        = 0
StartTime      = time()
StartDate      = date()
Message         = '(none)'
Tab             = d2c(9)
Esc = d2c(27)||'['

ProgramName    = Esc||'1;33m'||filespec('NAME', Spec)||Esc||'0m'
Version         = 'Version 0.92'
DateModified    = '(22/11/93)'

CurrentDir     = directory()


PortNo = 1            /* I use this CMD file on COM1, Task 1 */

/*      Perform test for error log file path test,       */
LogDir         = 'D:\BINKLEY\LOGS'

if directory(LogDir) <> LogDir then do
   say directory(LogDir)||Tab||LogDir
   say
   say Tab||' *******CANNOT CREATE LOG FILE*******'
   say Tab||' *******ABORTING '||ProgramName||'*******'
   exit(1)
end

/**\
 PATHS,
      NOTE TO USER: The function CheckPath('[Drive]:\[Path]') tests
                    for the existence of '[Drive]:\[Path]' and sets
                    the appropriate variable to it ONLY IF IT DOES EXIST.
                    Therefore, YOU must edit the '[Drive]:\[Path]' arguments
                    of each call to CheckPath to reflect YOUR hard disk.

               --MAKE A COPY OF THIS FILE BEFORE YOU EDIT IT--
**/
BinkleyDir      = CheckPath('D:\BINKLEY')
MaxDir          = CheckPath('D:\MAX')

FlagQueue = CheckQueue('BBS_CMD')

LogFile = LogDir||'\Bunyip'||PortNo||'.log'  

message = '* '||left(date(),Length(date())-5) time()||' SPAWNBBS s='||Speed
message = message||' b='||Baud||' t='||TimeLeft||' n=0'||PortNo||' e='||ExtInfo
call lineout LogFile, Message
call lineout LogFile

rc = Directory(MaxDir)

if Baud = '19200' then Baud = '300'

MaxParams =' -b'||Baud||' -p'||Port||' -t'||TimeLeft||' -s'||Speed
MaxParams = MaxParams||' -n0'||PortNo||' -l'||LogFile||' -z\pipe\bbs'||PortNo

'maxp.exe '||MaxParams

if rc = 12 | rc = 11 then call SendQMsg 'SQUISH'

call SendQMsg 'LCALL'

exit 0

/**\
       Sub-routine CheckPath
       This routine accepts a path argument and tests
       for its existance.
\**/
CheckPath: procedure
parse arg PathName

if directory(PathName) = '' then
   call ErrorProc 'Path error: '||pathname||' does not exist!'

return PathName
exit 1

/**\
       Sub-routine CheckQueue
       This routine accepts the name of the message queue used to
       pass flags to this program. The queue is created if needed and
       set as the current session message queue.
\**/
CheckQueue: procedure
parse arg QueueName

QueueTest      = RxQueue('CREATE', QueueName)

if QueueTest <> QueueName then
   rc   = RxQueue('DELETE', QueueTest)

OldQueue       = RxQueue('SET', QueueName)

return QueueName
exit 1

/**\
       SendQMsg
       Pushes a message into the queue
\**/
SendQMsg: procedure
parse arg Message

call RxQueue 'SET', 'BBS_CMD'
queue Message

call SysSleep 1

return

exit 1

/**\
       Sub-routine: ErrorProc
       This routine writes the passed argument to [name of program].LOG
\**/
ErrorProc: Procedure expose Spec LogDir Tab ProgramName CurrentDir
parse arg ErrorMsg

pName   = filespec('NAME', Spec)

tName   = left(pName, pos('.', pName) - 1)
ErrorLogFile = LogDir||'\'||tName||'.LOG'

rc      = beep(880, 1000)

say; say ' *******'||errormsg||'*******'
say; say ' LOGGING ERROR TO '||ErrorLogFile

call lineout ErrorLogFile, ErrorMsg||Tab||date()||Tab||time()
call lineout ErrorLogFile

say ' Terminating '||ProgramName

rc      = directory(CurrentDir)
exit(1)

